""" RichTopic upgrades
"""
import transaction
from Products.CMFCore.utils import getToolByName
from Products.ZCatalog.ProgressHandler import ZLogHandler

import logging
logger = logging.getLogger('Products.RichTopic.upgrades')

def migrate2attopic(context):
    """ Migrate RichTopic to ATTopic
    """
    catalog = getToolByName(context, 'portal_catalog')
    brains = catalog.unrestrictedSearchResults(portal_type='RichTopic')

    logger.info('Migrating RichTopic to ATTopic...')
    pghandler = ZLogHandler(100)
    pghandler.init('Upgrading RichTopic to Topic', len(brains))

    for index, brain in enumerate(brains):
        try:
            doc = brain.getObject()
        except ValueError:
            # To fix cataloged portal_factory entries
            logger.info('Skipped and uncatalog %s' % brain.getURL())
            catalog.uncatalog_object(catalog.getpath(brain.data_record_id_))
        doc.portal_type = 'Topic'
        if hasattr(doc, 'richContent'):
            text = getattr(doc, 'richContent')
            if callable(text):
                text = text()
            if text:
                doc.getField('text').getMutator(doc)(text)
            del doc.richContent
        if getattr(doc, 'layout', '') == 'richtopic_view':
            doc.layout = 'atct_topic_view'

        # XXX Catalog reindex doesn't work at this step. Please reindex later
        #     as a final step of migration.
        #catalog.reindexObject(doc, idxs=['portal_type', 'meta_type'])

        pghandler.report(index)
        if index % 100 == 0:
            transaction.savepoint(optimistic=True)

    ptypes = getToolByName(context, 'portal_types')
    if 'RichTopic' in ptypes.objectIds():
        ptypes.manage_delObjects(['RichTopic'])

    transaction.savepoint(optimistic=True)
    pghandler.finish()
    logger.info('Migrating RichTopic to ATTopic... DONE')

def removeBrokenBrains(context):
    """ Remove broken brains
    """
    catalog = getToolByName(context, 'portal_catalog')
    brains = catalog.unrestrictedSearchResults(portal_type='NewsletterTheme')

    logger.info('Removing broken brains...')

    for brain in brains:
        try:
            doc = brain.getObject()
        except ValueError:
            logger.info('Uncatalog %s' % brain.getURL())
            catalog.uncatalog_object(catalog.getpath(brain.data_record_id_))
            continue
            
        if 'portal_factory' in brain.getURL():
            logger.info('Uncatalog %s' % brain.getURL())
            catalog.uncatalog_object(catalog.getpath(brain.data_record_id_))

        transaction.savepoint(optimistic=True)

    logger.info('Removing broken brains... DONE')
